@@ -1,6 +1,7 @@
 #workflow
-from .workflow.executor import Executor, Node
-from .workflow.dependency import Dependency, Edge
+from .workflow.node import Executor, Service, Node
+from .workflow.edge import Dependency, Edge
+from .workflow.scaler import  ScalerTrigger, ScalerTriggerPrometheusMetadata, KedaSpec, HpaSpec
 from .workflow.workflow import Workflow
 #agent
 from .agent.sensor import Sensor, BaseTrigger, CronTrigger, IntervalTrigger, DateTrigger
